// SG12232C_sample.ino
//
// SG12232Cに文字を表示するサンプルプログラム
//

// ピン割り当ての定数宣言
#define A0_PIN  A3
#define CS1_PIN A0
#define CS2_PIN A1
#define E_PIN   A4
#define RW_PIN  A2
#define DB0_PIN 8
#define DB1_PIN 9
#define DB2_PIN 10
#define DB3_PIN 11
#define DB4_PIN 4
#define DB5_PIN 5
#define DB6_PIN 6
#define DB7_PIN 7

// WriteData/ReadData用コマンド定数宣言
#define DISPLAY_ON_COMMAND        0xaf
#define DISPLAY_OFF_COMMAND       0xae
#define DISPLAY_LINE_COMMAND      0xc0
#define PAGE_ADDRESS_COMMAND      0xb8
#define COLUMN_ADDRESS_COMMAND    0x00
#define STATIC_DRIVE_ON_COMMAND   0xa5
#define STATIC_DRIVE_OFF_COMMAND  0xa4
#define READ_MODIFY_WRITE_COMMAND 0xe0
#define END_COMMAND               0xee
#define RESET_COMMAND             0xe2

// WriteData/ReadData用の制御線状態定数宣言
#define CTRL_CS1     2 // CS1空間にアクセス
#define CTRL_CS2     4 // CS2空間にアクセス
#define CTRL_CS1_CS2 6 // CS1空間とCS2空間に同時に書き込みアクセス(読み込みアクセスはできない)
#define CTRL_COMMAND 0 // コマンドの書き込み、またはステータスの読み込み
#define CTRL_DATA    1 // データのアクセス
#define CTRL_CS1_COMMAND     (CTRL_CS1     | CTRL_COMMAND)
#define CTRL_CS1_DATA        (CTRL_CS1     | CTRL_DATA   )
#define CTRL_CS2_COMMAND     (CTRL_CS2     | CTRL_COMMAND)
#define CTRL_CS2_DATA        (CTRL_CS2     | CTRL_DATA   )
#define CTRL_CS1_CS2_COMMAND (CTRL_CS1_CS2 | CTRL_COMMAND)
#define CTRL_CS1_CS2_DATA    (CTRL_CS1_CS2 | CTRL_DATA   )

// データバスのピン割り当て用配列
static int DB[8] = { DB0_PIN, DB1_PIN, DB2_PIN, DB3_PIN, DB4_PIN, DB5_PIN, DB6_PIN, DB7_PIN };

// LCD制御用のピンの割り当てと、モード設定、初期値設定を行う
static void AssignPin()
{
  int i;
  
  // 制御信号線を出力に設定する
  pinMode(A0_PIN ,OUTPUT);
  pinMode(CS1_PIN,OUTPUT);
  pinMode(CS2_PIN,OUTPUT);
  pinMode(E_PIN  ,OUTPUT);
  pinMode(RW_PIN ,OUTPUT);
  
  // 制御信号線の初期値を設定する
  digitalWrite(CS1_PIN,HIGH);
  digitalWrite(CS2_PIN,HIGH);
  digitalWrite(E_PIN  ,LOW );
  digitalWrite(RW_PIN  ,LOW );

  // データバスをINPUT_PULLUPモードにする
  for(i=0; i<8; i++) {
    // 以下の2行は、Arduino IDE 1.0.1ならpinMode(DB[i],INPUT_PULLUP);に置き換えられる
    pinMode(DB[i],INPUT);
    digitalWrite(DB[i],HIGH);
  } // for i
} // AssignPin

// バスからデータやステータスを読む
static unsigned char ReadData(unsigned char ctrl)
{
  unsigned char result;
  int i;
  
  // CS1/CS2信号の制御
  if(ctrl&CTRL_CS1) digitalWrite(CS1_PIN,LOW);
  if(ctrl&CTRL_CS2) digitalWrite(CS2_PIN,LOW);

  // A0ピンの制御
  digitalWrite(A0_PIN,ctrl&CTRL_DATA);
  
  // R/Wピンの制御
  digitalWrite(RW_PIN,HIGH);
  
  // E信号の制御
  digitalWrite(E_PIN,HIGH);

  // データをバスから読む
  result=0;
  for(i=0; i<8; i++) {
    if(digitalRead(DB[i])==HIGH) result+=1<<i; 
  } // for i  
  
  // E信号の制御
  digitalWrite(E_PIN,LOW);

  // CS1/CS2信号の制御
  if(ctrl&CTRL_CS1) digitalWrite(CS1_PIN,HIGH);
  if(ctrl&CTRL_CS2) digitalWrite(CS2_PIN,HIGH);  
  
  return result;
} // ReadData

// バスにデータやコマンドを出力する
static void WriteData(unsigned char ctrl, unsigned char data)
{
  int i;
  
  // CS1/CS2信号の制御
  if(ctrl&CTRL_CS1) digitalWrite(CS1_PIN,LOW);
  if(ctrl&CTRL_CS2) digitalWrite(CS2_PIN,LOW);
  
  // A0ピンの制御
  digitalWrite(A0_PIN,ctrl&CTRL_DATA);
  
  // R/Wピンの制御
  digitalWrite(RW_PIN,LOW);
  
  // データバスを出力モードにする
  for(i=0; i<8; i++) {
    pinMode(DB[i],OUTPUT);
  } // for i
  
  // データバスにデータを出力する
  for(i=0; i<8; i++) {
    if((data>>i)&1) {
      digitalWrite(DB[i],HIGH);
    } else {
      digitalWrite(DB[i],LOW );
    } // if
  } // for i
  
  // E信号の制御
  digitalWrite(E_PIN,HIGH);
  digitalWrite(E_PIN,LOW );
  
  // データバスをINPUT_PULLUPモードにする
  for(i=0; i<8; i++) {
    // 以下の2行は、Arduino IDE 1.0.1ならpinMode(DB[i],INPUT_PULLUP);に置き換えられる
    pinMode(DB[i],INPUT);
    digitalWrite(DB[i],HIGH);
  } // for i

  // CS1/CS2信号の制御
  if(ctrl&CTRL_CS1) digitalWrite(CS1_PIN,HIGH);
  if(ctrl&CTRL_CS2) digitalWrite(CS2_PIN,HIGH);
} // WriteData

// LCDに初期化コマンドを送る
static void reset(void)
{
  while(ReadData(CTRL_CS1_COMMAND)&0x90 || ReadData(CTRL_CS2_COMMAND)&0x90); // ハードウェアリセットの終了を待つ(実際にはArduinoの起動が遅いので必要ない)
  WriteData(CTRL_CS1_CS2_COMMAND,RESET_COMMAND           );
  while(ReadData(CTRL_CS1_COMMAND)&0x90 || ReadData(CTRL_CS2_COMMAND)&0x90); // ソフトウェアリセットの終了を待つ
  WriteData(CTRL_CS1_CS2_COMMAND,STATIC_DRIVE_OFF_COMMAND);
  WriteData(CTRL_CS1_CS2_COMMAND,DISPLAY_ON_COMMAND      );
  WriteData(CTRL_CS1_CS2_COMMAND,DISPLAY_LINE_COMMAND    );
  WriteData(CTRL_CS1_CS2_COMMAND,PAGE_ADDRESS_COMMAND    );
  WriteData(CTRL_CS1_CS2_COMMAND,COLUMN_ADDRESS_COMMAND  );
} // reset

// 画面を消去する
static void ClearScreen(void)
{
  int page,column;
  
  for(page=0; page<4; page++) {
    // ページを設定する
    WriteData(CTRL_CS1_CS2_COMMAND,PAGE_ADDRESS_COMMAND+page);
    
    // カラムアドレスを0に設定する
    WriteData(CTRL_CS1_CS2_COMMAND,COLUMN_ADDRESS_COMMAND);
    
    // バスに0を出力する
    for(column=0; column<61; column++) {
      WriteData(CTRL_CS1_CS2_DATA,0);
    } // for column
  } // for page
} // ClearScreen

void setup()
{
  AssignPin();
  reset();
  ClearScreen();
  
  //画面に'A'の文字を表示する
  WriteData(CTRL_CS1_COMMAND,PAGE_ADDRESS_COMMAND  ); // ページアドレスを0にセット
  WriteData(CTRL_CS1_COMMAND,COLUMN_ADDRESS_COMMAND); // カラムアドレスを0にセット
  WriteData(CTRL_CS1_DATA,0x7e); // カラムアドレス0のデータを出力
  WriteData(CTRL_CS1_DATA,0x11); // カラムアドレス1のデータを出力
  WriteData(CTRL_CS1_DATA,0x11); // カラムアドレス2のデータを出力
  WriteData(CTRL_CS1_DATA,0x11); // カラムアドレス3のデータを出力
  WriteData(CTRL_CS1_DATA,0x7e); // カラムアドレス4のデータを出力 
} // setup

void loop()
{
} // loop

